@@ -1,16 +1,21 @@
 # -*- coding: latin-1 -*-
-# Copyright (c) 2008-2009 Michael Howitz
+# Copyright (c) 2008-2010 Michael Howitz
 # See also LICENSE.txt
-# $Id: addressbook.py 1089 2009-03-31 16:03:01Z mac $
+# $Id: addressbook.py 783 2010-01-09 14:28:06Z icemac $
 
+from icemac.addressbook.i18n import MessageFactory as _
+import icemac.addressbook.entities
+import icemac.addressbook.interfaces
+import icemac.addressbook.keyword
+import icemac.addressbook.utils
 import zc.catalog.catalogindex
 import zope.app.appsetup.bootstrap
 import zope.app.authentication.authentication
 import zope.app.authentication.interfaces
 import zope.app.authentication.principalfolder
+import zope.authentication.interfaces
 import zope.catalog.catalog
 import zope.catalog.interfaces
-import zope.authentication.interfaces
 import zope.component
 import zope.container.btree
 import zope.container.interfaces
@@ -21,9 +26,6 @@ import zope.location
 import zope.location.interfaces
 import zope.schema.fieldproperty
 import zope.site.site
-import icemac.addressbook.interfaces
-import icemac.addressbook.keyword
-import icemac.addressbook.utils
 
 
 class AddressBook(zope.container.btree.BTreeContainer,
@@ -32,18 +34,34 @@ class AddressBook(zope.container.btree.BTreeContainer,
 
     zope.interface.implements(icemac.addressbook.interfaces.IAddressBook)
 
+    entities = None
+    importer = None
     keywords = None
     principals = None
 
     title = zope.schema.fieldproperty.FieldProperty(
         icemac.addressbook.interfaces.IAddressBook['title'])
-    notes = zope.schema.fieldproperty.FieldProperty(
-        icemac.addressbook.interfaces.IPerson['notes'])
+
+
+address_book_entity = icemac.addressbook.entities.create_entity(
+    _(u'address book'), icemac.addressbook.interfaces.IAddressBook, AddressBook)
+
+
+def create_and_register(addressbook, attrib_name, class_, interface, name=''):
+    """Create an object on an attribute and register it as local utility."""
+    if getattr(addressbook, attrib_name) is not None:
+        return
+    site_mgr = addressbook.getSiteManager()
+    obj = icemac.addressbook.utils.create_obj(class_)
+    setattr(addressbook, attrib_name, obj)
+    zope.location.locate(obj, addressbook, '++attribute++'+attrib_name)
+    site_mgr.registerUtility(obj, interface, name=name)
+
 
 @zope.component.adapter(
     icemac.addressbook.interfaces.IAddressBook,
     zope.container.interfaces.IObjectAddedEvent)
-def create_address_book_infrastructure(addressbook, event):
+def create_address_book_infrastructure(addressbook, event=None):
     """Create initial infrastructure or update existing infrastructure to
     current requirements (using generation)."""
     # add site manager
@@ -51,16 +69,23 @@ def create_address_book_infrastructure(addressbook, event):
         site_manager = zope.site.site.LocalSiteManager(addressbook)
         addressbook.setSiteManager(site_manager)
 
-    site_manager = addressbook.getSiteManager()
-
     # add keywords container
-    if addressbook.keywords is None:
-        keywords = icemac.addressbook.utils.create_obj(
-            icemac.addressbook.keyword.KeywordContainer)
-        addressbook.keywords = keywords
-        zope.location.locate(keywords, addressbook, '++attribute++keywords')
-        site_manager.registerUtility(
-            keywords, icemac.addressbook.interfaces.IKeywords)
+    create_and_register(
+        addressbook, 'keywords', icemac.addressbook.keyword.KeywordContainer,
+        icemac.addressbook.interfaces.IKeywords)
+
+    # add principals folder
+    create_and_register(
+        addressbook, 'principals',
+        zope.app.authentication.principalfolder.PrincipalFolder,
+        zope.app.authentication.interfaces.IAuthenticatorPlugin,
+        name=u'icemac.addressbook.principals')
+
+    # add entities utility
+    create_and_register(
+        addressbook, 'entities',
+        icemac.addressbook.entities.PersistentEntities,
+        icemac.addressbook.interfaces.IEntities)
 
     add_more_addressbook_infrastructure(addressbook, addressbook)
 
@@ -107,7 +132,7 @@ def add_more_addressbook_infrastructure(addressbook):
     # indexes
     if 'keywords' not in catalog:
         catalog['keywords'] = zc.catalog.catalogindex.SetIndex(
-            'get_titles', icemac.addressbook.interfaces.IKeywords,
+            'get_titles', icemac.addressbook.interfaces.IKeywordTitles,
             field_callable=True)
 
     # authenticator (PAU)
@@ -118,15 +143,4 @@ def add_more_addressbook_infrastructure(addressbook):
             zope.app.authentication.authentication.PluggableAuthentication)
         pau.credentialsPlugins = (u'No Challenge if Authenticated',
                                   u'Session Credentials',)
-
-    # principals folder
-    if addressbook.principals is None:
-        principals = icemac.addressbook.utils.create_obj(
-            zope.app.authentication.principalfolder.PrincipalFolder)
-        addressbook.principals = principals
-        zope.location.locate(principals, addressbook, '++attribute++principals')
-        site_manager = addressbook.getSiteManager()
-        site_manager.registerUtility(
-            principals, zope.app.authentication.interfaces.IAuthenticatorPlugin,
-            name=u'icemac.addressbook.principals')
-    pau.authenticatorPlugins = (u'icemac.addressbook.principals',)
+        pau.authenticatorPlugins = (u'icemac.addressbook.principals',)
