@@ -1,12 +1,13 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2009 Michael Howitz
+# Copyright (c) 2009-2010 Michael Howitz
 # See also LICENSE.txt
 
 import StringIO
 import doctest
+import logging
 import sys
 import zc.buildout.testing
-import logging
+import zope.testing.doctest
 
 
 def DocFileSuite(*args, **kw):
@@ -43,7 +44,7 @@ def DocFileSuite(*args, **kw):
     kw['tearDown'] = tearDown
     kw['optionflags'] = (doctest.ELLIPSIS | doctest.NORMALIZE_WHITESPACE)
     kw['globs'] = dict(call_with_user_input=call_with_user_input)
-    return doctest.DocFileSuite(*args, **kw)
+    return zope.testing.doctest.DocFileSuite(*args, **kw)
 
 def test_suite():
     return DocFileSuite('../install.txt')
